
// MouseController.java
// Andrew Davison, August 2012, ad@fivedots.coe.psu.ac.th

/* Report changes to the components in the mouse controller 
   with the specufied index.
   Uses JInput (http://java.net/projects/jinput)

   Each call to update() polls the controller and then updates the values
   for its three buttons, (x,y) movement, and wheel rotation.
*/


import net.java.games.input.*;
import net.java.games.input.Component.Identifier;
import net.java.games.input.Controller.Type;
import net.java.games.input.ControllerEnvironment;


public class MouseController
{
  private Mouse mouseCtrl = null;

  /* these global booleans allow me to report button presses and releases
     only when they first occur */
  private boolean isLeftPressed = false;
  private boolean isMiddlePressed = false;
  private boolean isRightPressed = false;



  public MouseController(int id)
  {
    ControllerEnvironment ce = ControllerEnvironment.getDefaultEnvironment();

    Controller[] ca = ce.getControllers();
    if (ca.length == 0) {
      System.out.println("No controllers found");
      System.exit(0);
    }

    if ((id < 0) || (id >= ca.length)) {
      System.out.println("Supplied index out of range (0-" + (ca.length-1) + ")");
      System.exit(0);
    }

    Controller.Type type = ca[id].getType();
    if (type != Controller.Type.MOUSE) {
      System.out.println("Controller[" + id + "] is not a mouse");
      System.exit(0);
    }

    mouseCtrl = (Mouse) ca[id];
    System.out.println("ID: " + id + "; name: \"" + mouseCtrl.getName() + "\"");
  }  // end of MouseController()



  // ----------------- updating data ------------------


  public boolean update()
  /* update the component values in the mouse by polling, 
     then report their values
  */
  {  
    if (!mouseCtrl.poll())
      return false;

    isLeftPressed = updatePress(mouseCtrl.getLeft(), isLeftPressed);
    isMiddlePressed = updatePress(mouseCtrl.getMiddle(), isMiddlePressed);
    isRightPressed = updatePress(mouseCtrl.getRight(), isRightPressed);

    showMove(mouseCtrl.getX());
    showMove(mouseCtrl.getY());
    showMove(mouseCtrl.getWheel());

    return true;
  }  // end of update()



  private boolean updatePress(Component button, boolean isPressed)
  /* update the isPressed boolean for the button if the button has 
     changed its state (pressed/released) 
  */
  {  
    if (button == null)
      return isPressed;    // no change

    String name = button.getName();
    float val = button.getPollData();

    if (isPressed && (val == 0)) {  // button released
      isPressed = false;
      System.out.println(name + ": released"); 
    }
    else if (!isPressed && (val == 1)) {  // button pressed
      isPressed = true;
      System.out.println(name + ": pressed");
    }

    return isPressed;
  }  // end of updatePress()



  private void showMove(Component mover)
  // the mover may be the x-, y-, or wheel component of the mouse
  {  
    if (mover != null) {
      int step = (int) mover.getPollData();
      if (step != 0)   // don't report 0 (i.e. no movement)
        System.out.println(mover.getName() + ": " + step);
    }
  }  // end of showMove()


}  // end of MouseController class
