
// FingerTipController.java
// Andrew Davison, August 2012, ad@fivedots.coe.psu.ac.th

/* A finger tip controller is a simple type of JInput mouse controller
   which monitors only the left button presses and the (x, y) moves.

   The idea is that 'finger' movement is implemented as mouse movement,
   and 'finger' presses as the pressing of the left button.

   To help with gesture interpretation in MultitouchPanel, each finger tip
   stores the current and previous button press status and (x, y) position
   based on when the mouse was polled.

   The controller also maintains two images for the unpressed and pressed finger
   tips, one of which is drawn at the current (x,y) position when draw() is called.
*/

import java.awt.*;
import java.io.*;
import java.awt.image.*;

import net.java.games.input.Component;
import net.java.games.input.Mouse;


public class FingerTipController
{
  private static final String TIP_IMAGE = "blueFingerTip.png";
  private static final String PRESSED_IMAGE = "redFingerTip.png";

  private static final int TIP_OFFSET = 100; // start posn info for finger tip images


  private Mouse mouseCtrl = null;
  private Component leftBut, xMove, yMove;
     // finger movement is implemented as mouse movement
     // finger pressing is implemented as left button pressing

  private Point prevLoc, location;
  private boolean prevPressed = false;
  private boolean isPressed = false;

  private BufferedImage tipIm, pressedTipIm;
  private int pWidth, pHeight, tipWidth, tipHeight;



  public FingerTipController(Mouse mc, boolean isLeftTip, int pWidth, int pHeight)
  {
    mouseCtrl = mc;
    System.out.print( (isLeftTip ? "Left " : "Right "));
    System.out.println("finger tip assigned to \"" + mouseCtrl.getName() + "\"");

    this.pWidth = pWidth;
    this.pHeight = pHeight;

    // initialize finger tip images
    tipIm = ImageUtils.loadImage(TIP_IMAGE);
    pressedTipIm = ImageUtils.loadImage(PRESSED_IMAGE);
    tipWidth = tipIm.getWidth();
    tipHeight = tipIm.getHeight();

    if (isLeftTip)
      location = new Point((TIP_OFFSET + tipWidth/2), (pHeight/2));
    else  // right tip
      location = new Point( (pWidth - TIP_OFFSET - tipWidth/2), (pHeight/2));
    prevLoc = new Point(location.x, location.y);

    leftBut = mouseCtrl.getLeft();
    xMove = mouseCtrl.getX();
    yMove = mouseCtrl.getY();
  }  // end of FingerTipController()


  public boolean isPressed()
  {  return isPressed;  }

  public boolean isPrevPressed()
  {  return prevPressed;  }


  public Point getLocation()
  {  return new Point(location.x, location.y);  }

  public Point getPrevLoc()
  {  return new Point(prevLoc.x, prevLoc.y);  }

  // ----------------- updating data ------------------


  public boolean update()
  // update the component values in the mouse
  {  
    boolean isValid = mouseCtrl.poll(); 
    if (!isValid)
      return false;

    updatePressed();
    updatePosition();
    return true;
  }  // end of update()



  private void updatePressed()
  {  
    prevPressed = isPressed;

    float val = leftBut.getPollData();
    if (isPressed && (val == 0))  // button released
      isPressed = false;
    else if (!isPressed && (val == 1))  // button pressed
      isPressed = true;
  }  // end of updatePressed()




  private void updatePosition()
  {  
    prevLoc = new Point(location.x, location.y);

    int xNew = location.x + (int) xMove.getPollData();
    if (xNew < 0)
      xNew = 0;
    else if (xNew >= pWidth)
      xNew = pWidth-1;

    int yNew = location.y + (int) yMove.getPollData();
    if (yNew < 0)
      yNew = 0;
    else if (yNew >= pHeight)
      yNew = pHeight-1;

    location.setLocation(xNew, yNew);
  }  // end of updatePosition()



  public void draw(Graphics g)
  {
    int x = location.x - tipWidth/2;
    int y = location.y - tipHeight/2;

    if (isPressed())
      g.drawImage(pressedTipIm, x, y, null);
    else
      g.drawImage(tipIm, x, y, null);
  }  // end of draw()

}  // end of FingerTipController class
