
// ImageHolder.java
// Andrew Davison, August 2012, ad@fivedots.coe.psu.ac.th

/* An ImageHolder object represents an image which can be moved,
   scaled, and rotated by finger tips.

  The image is initially positioned at a random location on the screen,
  with its center position stored in the location point. 

  The border of a loaded image is padded with transparent pixels so that
  it becomes big enough so that the image can be rotated around its center
  without clipping any visible pixels. This simplifies the rotation code
  at the expense of increasing the image size by double or more!

  It is not possible to move the location point off the screen. 

  The current angle (in degrees) and scale factor are stored, so that if
  they return to 0 and 1 respectively, the image is 'refreshed' by being
  replaced with the original loaded image.
*/

import java.awt.*;
import java.util.*;
import java.awt.image.*;
import java.awt.geom.*;


public class ImageHolder
{
  private int pWidth, pHeight;         // panel size

  private BufferedImage origIm, im;    // original and current images
  private Point centerLoc;   // center point of current image
  private int currAngle = 0;    // in degrees
  private double currScale = 1.0;



  public ImageHolder(String fnm, int pWidth, int pHeight)
  {
    this.pWidth = pWidth;
    this.pHeight = pHeight;

    origIm = ImageUtils.padBorders( ImageUtils.loadImage(fnm));
       /* padBorders() enlarges the image by padding its borders
          with transparent pixels. This makes it simpler to rotate the
          image later */
    int w = origIm.getWidth();
    int h = origIm.getHeight();
    im = origIm;

    Random r = new Random();
    int x = r.nextInt(pWidth-w);
    int y = r.nextInt(pHeight-h);

    centerLoc = new Point(w/2+x, h/2+y);
  }  // end of ImageHolder()



  public boolean contains(Point pt)
  // does this image contain the screen coordinate in pt?
  {
    // calculate (x,y) relative to image coords
    int x = pt.x - (centerLoc.x - im.getWidth()/2);
    int y = pt.y - (centerLoc.y - im.getHeight()/2);
    return ImageUtils.onVisiblePixel(im, x, y);
           /* (x,y) is only on an image if it is over a visible pixel 
              of the image */
  }  // end of contains()



  public void move(int dx, int dy)
  // move the image by dx and dy offsets
  {
    if ((dx == 0) && (dy == 0)) // no change
      return;
    Point movePt = restrictPosition(centerLoc.x + dx, centerLoc.y + dy);
    centerLoc.setLocation(movePt);   // update the current position of the image
  }  // end of move()


  private Point restrictPosition(int x, int y)
  /* calculate the new center as (x,y) but don't allow the
     point to move off the screen */
  {
    if (x < 0)
      x = 0;
    else if (x >= pWidth)
      x = pWidth-1;

    if (y < 0)
      y = 0;
    else if (y >= pHeight)
      y = pHeight-1;

    return new Point(x, y);
  }  // end of restrictPosition()



  public void rotate(Point pivot, double angle)
  /* Rotate this image by angle radians around the pivot point.
     The rotation is done in two steps -- the center point
     of the image is rotated around the pivot by angle radians, 
     and the image is also rotated around its center point by angle
     radians.

     Note that the image always rotates around its center, not around
     the tip point. This simplifies the ImageUtils.rotate() method.
  */
  {
    // rotate the center point of the image around the pivot
    Point rotatedLoc = rotatePt(centerLoc, pivot, angle);
    Point newLoc = restrictPosition(rotatedLoc.x, rotatedLoc.y);
    centerLoc.setLocation(newLoc);

    int degAngle = (int)Math.round(Math.toDegrees(angle));
    currAngle = (currAngle + degAngle)%360;

    // rotate the image around its  center point
    if ((currAngle == 0) && (currScale == 1))    // reuse original image if unscaled
      im = origIm;
    else
      im = ImageUtils.rotate(im, angle);
  }  // end of rotate()



  private Point rotatePt(Point pt, Point pivot, double angle)
  //  rotate the pt point around a given pivot point
  {
    AffineTransform at =  AffineTransform.getRotateInstance(angle, pivot.x, pivot.y);
    Point2D pt2 = at.transform(pt, null);  
    return new Point((int)pt2.getX(), (int)pt2.getY());   // truncate to an integer point
  }  // end of rotatePt()



  public void scale(double scale)
  // scale the image by the specified scale factor (within certain limits)
  {
    // scaling does not change the center point of the image

    if (scale == 1.0)  // no change in scale
      return;
    else if (scale > 1.0) {
      if (im.getHeight()*scale >= pHeight)  // new height too big
        scale = ((double)pHeight-1)/im.getHeight();
      else if (im.getWidth()*scale >= pWidth)  // new width too big
        scale = ((double)pWidth-1)/im.getWidth();
      im = ImageUtils.scale(im, scale);
      currScale *= scale;
    }
    else {     // scale < 1.0
      if (im.getWidth()*scale <= origIm.getWidth()) {  // too small
        im = ImageUtils.rotate(origIm, Math.toRadians(currAngle));   
                                                 // use original image, rotated
        currScale = 1;   // back to original size
      }
      else {
        im = ImageUtils.scale(im, scale);
        currScale *= scale;
      }
    }
  }  // end of scale()



  public void draw(Graphics g)
  // draw the image centered at the centerLoc point
  {
    int x = centerLoc.x - im.getWidth()/2;
    int y = centerLoc.y - im.getHeight()/2;
    g.drawImage(im, x, y, null);
  }  // end of draw()

}  // end of ImageHolder class
