
// ImagesManager.java
// Andrew Davison, August 2012, ad@fivedots.coe.psu.ac.th

/* Manager the on-screen images. Each image is represented by
   an ImageHolder object.

   Management consists of passing a move, scale, or rotation command
   to the correct image based on a tip point position. When an image is
   selected, it is moved to the end of the images list, which means
   that it will be drawn on top of the other images.

   The draw() method called by the GUI thread is synchronized.
   This means that it is not possible for the images list to be changed by 
   findImage() at the same time, because it is also synchronized.
*/

import java.util.*;
import java.awt.*;


public class ImagesManager
{
  private static final String[] imFnms = {
                    "cat2.png", "domestic-cat.png", "greeneyes.png",
                    "old-cat-sleeping.png", "SiameseCat.png"
                  };     // all in the images/ directory


  private ArrayList<ImageHolder> images;
      // these images are drawn in list order, so the last one is
      // drawn on top of the others



  public ImagesManager(int pWidth, int pHeight)
  // pWidth and pHeight are the width and height of the panel (i.e. the screen)
  { 
    images = new ArrayList<ImageHolder>();
    for(String fnm : imFnms)
      images.add( new ImageHolder(fnm, pWidth, pHeight));
  }  // end of ImagesManager()



  public void move(Point tipPt, int dx, int dy)
  // move the image at tipPt by dx and dy offsets
  {
    ImageHolder imHolder = findImage(tipPt);
    if (imHolder != null)
      imHolder.move(dx, dy);
  }  // end of move()



  public synchronized ImageHolder findImage(Point tipPt)
  /* find the image at the tip point, and move it to the 'top' 
     (the end of the list) */
  {
    // go through images from 'top' to 'bottom'
    int i = images.size()-1;
    while (i >= 0) {
      if (images.get(i).contains(tipPt))
        break;
      i--;
    }

    if (i >= 0) {    // move found image to the 'top'
      ImageHolder imH = images.remove(i);
      images.add(imH);
      return imH;
    }
    else
      return null;
  }  // end of findImage()


  public synchronized boolean overImage(Point tipPt)
  /* is the tip point over an image? 
     This method doesn't change the list */
  {
    for(ImageHolder imH : images)
      if (imH.contains(tipPt))
        return true;
    return false;
  }  // end of overImage()



  public void rotate(Point tipPt, Point pivot, double angle)
  /* rotate the image at the tip point by angle radians around 
     the pivot point */
  {
    ImageHolder imHolder = findImage(tipPt);
    if (imHolder != null)
      imHolder.rotate(pivot, angle);
  }  // end of rotate()



  public void scale(Point tipPt, double scale)
  // scale the image at the tip point
  {
    ImageHolder imHolder = findImage(tipPt);
    if (imHolder != null)
      imHolder.scale(scale);
  }  // end of scale()



  public synchronized void draw(Graphics g)
  /* draw all the images in the list; the last image is drawn last
     which puts it on top of the others */
  {
    for(ImageHolder imHolder : images)
      imHolder.draw(g);
  }  // end of draw()

}  // end of ImagesManager class
