
// MultitouchPanel.java
// Andrew Davison, August 2012, ad@fivedots.coe.psu.ac.th

/* The drawing surface, which covers the entire screen.

   The run() methods shows the standard coding style. The thread
   enters a  3-element cycle:
      update the finfer tip controllers and images / 
      draw the images / sleep

   The sleep time tries to keep each cycle close to DELAY ms.

   The finger tip controllers are initialized with the first
   two IDs collected by a search for all the mouse controllers

   The app ends when:
     * the user presses ESC, q, ctrl-c;
*/


import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;


import net.java.games.input.*;
import net.java.games.input.Controller.Type;



public class MultitouchPanel extends JPanel implements Runnable
{
  private static final int DELAY = 25;  // ms  (polling interval)


  private boolean isRunning = false;   // used to stop the loop

  private FingerTipController leftTip, rightTip;
  private ImagesManager imsMan;



  public MultitouchPanel()
  {
    // set panel/game dimensions to be those of the entire screen
    Dimension scrDim = Toolkit.getDefaultToolkit().getScreenSize();   // screen size
    setPreferredSize(scrDim); 

    int pWidth = scrDim.width;
    int pHeight = scrDim.height;
    // System.out.println("Panel (w,h) : (" + pWidth + ", " + pHeight + ")");

    // listen for key presses
    setFocusable(true);
    requestFocus();    // the JPanel now has focus, so receives key events
    initKeyListener();

    initFingerTips(pWidth, pHeight);
    imsMan = new ImagesManager(pWidth, pHeight);

    new Thread(this).start();   // start updating the panel
  } // end of MultitouchPanel()



  private void initKeyListener()
  // define keys for stopping
  {
    addKeyListener( new KeyAdapter() {
       public void keyPressed(KeyEvent e)
       { 
         int keyCode = e.getKeyCode();
         if ((keyCode == KeyEvent.VK_ESCAPE) || (keyCode == KeyEvent.VK_Q) ||
             ((keyCode == KeyEvent.VK_C) && e.isControlDown()) )
                // ESC, q, ctrl-c to stop isRunning 
           isRunning = false;
       }
     });
  }  // end of initKeyListener()



  private void initFingerTips(int pWidth, int pHeight)
  /* Initialize the left and right finger tip controllers using JInput 
     mouse controlers, which are specified using their controller ID.
  */
  {
    ControllerEnvironment ce = ControllerEnvironment.getDefaultEnvironment();
    Controller[] ca = ce.getControllers();
    if (ca.length == 0) {
      System.out.println("No controllers found");
      System.exit(0);
    }

    // collect the IDs of all the mouse controllers
    int[] mouseIDs = new int[ca.length];
    int mouseCount = 0;
    System.out.println("Mouse Controllers:");
    for (int i = 0; i < ca.length; i++) {
      if (ca[i].getType() == Controller.Type.MOUSE) {
        System.out.println("  ID " + i + "; \"" + ca[i].getName() +"\"");
        mouseIDs[mouseCount++] = i;
      }
    }

    if (mouseCount < 2) {
      System.out.println("Not enough found (" + mouseCount + "); 2 needed");
      System.exit(0);
    }

    // left finger tip
    int idx = mouseIDs[0];
    System.out.println("\nInitializing mouse ID " + idx + "...");
    leftTip = new FingerTipController((Mouse)ca[idx], true, pWidth, pHeight);

    // right finger tip
    idx = mouseIDs[1];
    System.out.println("\nInitializing mouse ID " + idx + "...");
    rightTip = new FingerTipController((Mouse)ca[idx], false, pWidth, pHeight);
  }  // end of initFingerTips()




  // -------------------------------------------


  public void run()
  /* Implements the app loop:
        update the controllers and images / draw  / sleep
     The sleep time tries to keep each cycle close to DELAY ms.
  */
  {
    long duration;
    isRunning = true;

    while(isRunning) {
      long startTime = System.currentTimeMillis();
      update();
      duration = System.currentTimeMillis() - startTime;
      // System.out.println(" update duration: " + duration);
      repaint();

      if (duration < DELAY) {
        try {
          Thread.sleep(DELAY-duration);  // wait until DELAY time has passed
        } 
        catch (Exception ex) {}
      }
    }
    System.exit(0);
  }  // end of run()



  public void closeDown() 
  {  isRunning = false; } 



  private void update()
  /* update the finger tip controllers, and then determine how they
     affect the images on screen */
  {  
    // update the finger tip controllers
    if (!leftTip.update()) {
      System.out.println("Left fingertip Controller no longer valid");
      System.exit(0);
    }

    if (!rightTip.update()) {
      System.out.println("Right fingertip Controller no longer valid");
      System.exit(0);
    }

    // get current and previous finger tip locations and pressed statuses
    Point prevLeftLoc = leftTip.getPrevLoc();
    Point leftLoc = leftTip.getLocation();
    Point prevRightLoc = rightTip.getPrevLoc();
    Point rightLoc = rightTip.getLocation();

    boolean prevLeftPressed = leftTip.isPrevPressed();
    boolean leftPressed = leftTip.isPressed();
    boolean prevRightPressed = rightTip.isPrevPressed();
    boolean rightPressed  = rightTip.isPressed(); 

    /* are the finger tips positioned to mean a move,
       double move (i.e. move two images at once), scale, or rotation?
    */
    if (leftPressed && !rightPressed){    // possible left tip move
      int dx = leftLoc.x - prevLeftLoc.x;
      int dy = leftLoc.y - prevLeftLoc.y;
      imsMan.move(prevLeftLoc, dx, dy);
    }
    else if (!leftPressed && rightPressed) {   // possible right tip move
      int dx = rightLoc.x - prevRightLoc.x;
      int dy = rightLoc.y - prevRightLoc.y;
      // System.out.println("move: " + dx + ", " + dy);
      imsMan.move(prevRightLoc, dx, dy);
    }

    // check for scale, rotation, double move
    if (leftPressed && rightPressed && prevLeftPressed && prevRightPressed) {
      if (imsMan.overImage(prevLeftLoc) &&
           (imsMan.findImage(prevLeftLoc) == 
            imsMan.findImage(prevRightLoc))) {     // tips on same image, so a scaling
        double prevDist = Math.abs( prevLeftLoc.distance(prevRightLoc));
        double currDist = Math.abs( leftLoc.distance(rightLoc));
        if ((prevDist != 0) && (currDist != 0))
          imsMan.scale(prevLeftLoc, currDist/prevDist);
      }
      else {   // tips not on same image, so maybe a rotation or double move
        if (imsMan.overImage(prevLeftLoc) && !imsMan.overImage(prevRightLoc)) {   
          // rotation of left tip using right tip as screen pivot
          double angle = angleBetween(rightLoc, leftLoc, prevLeftLoc);
          imsMan.rotate(prevLeftLoc, rightLoc, angle);
        }
        else if (imsMan.overImage(prevRightLoc) && !imsMan.overImage(prevLeftLoc)) {   
          // rotation of right tip using left tip as screen pivot
          double angle = angleBetween(leftLoc, rightLoc, prevRightLoc);
          imsMan.rotate(prevRightLoc, leftLoc, angle);
        }
        else {   // tips pressed on two different images, so move both
          int dx = leftLoc.x - prevLeftLoc.x;
          int dy = leftLoc.y - prevLeftLoc.y;
          imsMan.move(prevLeftLoc, dx, dy);   // move left
          dx = rightLoc.x - prevRightLoc.x;
          dy = rightLoc.y - prevRightLoc.y;
          imsMan.move(prevRightLoc, dx, dy);   // move right
        }
      }
    }
  }  // end of update()


  private double angleBetween(Point pivot, Point curr, Point prev) 
  /* return the angle between prev and curr points relative to the 
     pivot point */
  {
    return Math.atan2(curr.x - pivot.x, -(curr.y - pivot.y)) -
           Math.atan2(prev.x - pivot.x, -(prev.y - pivot.y));
  }


  // --------------------- rendering ------------------------------


  public void paintComponent(Graphics g)
  { 
    super.paintComponent(g);
    imsMan.draw(g);        // draw images
    leftTip.draw(g);       // draw fingertips
    rightTip.draw(g);
  } // end of paintComponent()


} // end of MultitouchPanel class
